package cn.com.qmth.stmms.ms.admin.service;

import java.awt.image.BufferedImage;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Date;
import java.util.List;

import javax.imageio.ImageIO;

import org.apache.commons.codec.digest.DigestUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import cn.com.qmth.stmms.ms.admin.dto.StudentDTO;
import cn.com.qmth.stmms.ms.commons.config.ImageCompressionConfig;
import cn.com.qmth.stmms.ms.commons.config.SystemConfig;
import cn.com.qmth.stmms.ms.commons.utils.excel.ExcelError;
import cn.com.qmth.stmms.ms.commons.utils.excel.ExcelReader;
import cn.com.qmth.stmms.ms.commons.utils.excel.ExcelReaderHandle;
import cn.com.qmth.stmms.ms.commons.utils.image.ImageCompression;
import cn.com.qmth.stmms.ms.core.domain.ExamQuestion;
import cn.com.qmth.stmms.ms.core.domain.Paper;
import cn.com.qmth.stmms.ms.core.domain.Student;
import cn.com.qmth.stmms.ms.core.repository.ExamQuestionRepo;
import cn.com.qmth.stmms.ms.core.repository.MarkTaskRepo;
import cn.com.qmth.stmms.ms.core.repository.PaperRepo;
import cn.com.qmth.stmms.ms.core.repository.StudentRepo;
import cn.com.qmth.stmms.ms.core.vo.Subject;

/**
 * 数据上传服务 Created by zhengmin on 2016/11/18.
 */
@Service
public class DataUploadService {

    @Autowired
    private StudentRepo studentRepo;

    @Autowired
    private ExamQuestionRepo examQuestionRepo;

    @Autowired
    private PaperRepo paperRepo;

    @Autowired
    private SystemConfig systemConfig;

    @Autowired
    private ImageCompressionConfig compressionConfig;

    @Autowired
    private MarkTaskRepo markTaskRepo;

    /**
     * 上传考生试卷数据
     *
     * @param studentId 考生id
     * @param subject   科目
     * @param in        图片文件输入流
     * @param isManual  是否手工绑定
     * @throws IOException
     */
    @Transactional
    public void uploadPaper(Long studentId, Subject subject, InputStream in, boolean isManual) throws IOException {
        Student student = studentRepo.findOne(studentId);
        String savePath = systemConfig.getImageDir() + File.separator + student.getWorkId() + File.separator + subject
                + File.separator + student.getAreaCode();
        File out = new File(savePath);
        if (!out.exists()) {
            out.mkdirs();
        }
        File outFile = new File(savePath + File.separator + student.getExamNumber() + ".jpg");
        if (!outFile.exists()) {
            outFile.createNewFile();
        }
        OutputStream output = new FileOutputStream(outFile);
        byte[] buffer = new byte[in.available()];
        in.read(buffer);
        output.write(buffer);
        output.close();

        // 生成缩略图
        String thumbDir = systemConfig.getThumbDir() + File.separator + student.getWorkId() + File.separator + subject
                + File.separator + student.getAreaCode();
        File thumb = new File(thumbDir);
        if (!thumb.exists()) {
            thumb.mkdirs();
        }
        BufferedImage bufferedImage = ImageCompression.compress(outFile, compressionConfig);
        String thumbFileName = thumbDir + File.separator + student.getExamNumber() + ".jpg";
        ImageIO.write(bufferedImage, "jpg", new File(thumbFileName));

        Paper exist = paperRepo.findByWorkIdAndSubjectAndExamNumber(student.getWorkId(), subject,
                student.getExamNumber());
        String sheetMD5 = DigestUtils.md5Hex(in);
        String sliceMD5 = DigestUtils.md5Hex(new FileInputStream(thumbFileName));
        if (exist != null) {
            exist.setManual(isManual);
            exist.setUploadedOn(new Date());
            exist.setSheetMD5(sheetMD5);
            exist.setSliceMD5(sliceMD5);
            paperRepo.save(exist);
        } else {
            ExamQuestion examQuestion = examQuestionRepo.findByWorkIdAndSubjectAndAreaCode(student.getWorkId(), subject,
                    student.getAreaCode());
            Paper paper = new Paper(student.getWorkId(), null, subject, examQuestion, student, false);
            paper.setSheetMD5(sheetMD5);
            paper.setSliceMD5(sliceMD5);
            paperRepo.save(paper);
            // 更新科目上传状态
            /**
             * Long count =
             * paperRepo.countByWorkIdAndExamNumber(student.getWorkId(),student.
             * getExamNumber()); if(count == Subject.values().length){
             * student.setUpload(true); studentRepo.save(student); }
             */
            synchronized (this) {
                String uploadStatus = student.getUploadStatus();
                String replace = subject.toString() + ":1";
                String regx = subject.toString() + ":[0-1]";
                String newStatus = uploadStatus.replaceFirst(regx, replace);
                student.setUploadStatus(newStatus);
                student.setAbsent(false);
                studentRepo.save(student);
            }
        }
    }

    @Transactional
    public List<ExcelError> uploadStudents(Long workId, final boolean isAbsent, InputStream inputStream) {
        ExcelReader excelReader = new ExcelReader(StudentDTO.class);
        List<ExcelError> excelErrors = excelReader.reader(inputStream, new ExcelReaderHandle() {

            @Override
            public ExcelError handle(Object obj) {
                StudentDTO dto = (StudentDTO) obj;

                String examNumber = dto.getExamNumber();
                //如果没有设置区域代码，用准考证前2位作为区域代码
                String areaCode = null;
                if (dto.getAreaCode() == null || dto.getAreaCode().isEmpty()) {
                    areaCode = examNumber.substring(0, 2);
                    dto.setAreaCode(areaCode);
                }

                long isExist = examQuestionRepo.countByWorkIdAndAreaCode(workId, dto.getAreaCode());
                if (isExist == 0) {
                    for (Subject subject : Subject.values()) {
                        ExamQuestion question = new ExamQuestion(dto.getAreaCode(), subject, workId, dto.getAreaName());
                        examQuestionRepo.save(question);
                    }
                }
                Student student = studentRepo.findByWorkIdAndExamNumber(workId, dto.getExamNumber());
                if (student == null && !isAbsent) {
                    student = new Student(dto.getName(), dto.getExamNumber(), dto.getAreaName(), dto.getAreaCode(),
                            workId, dto.getExamRoom(), dto.getSourceName());
                    studentRepo.save(student);
                } else if (student != null) {
                    if (isAbsent) {
                        student.setAbsent(isAbsent);
                        student.setUploadStatus("SX:0,SC:0,SM:0");

                        /**
                         * 查询该考生所有paper,若paper无任务则删除，有任务就保留
                         */
                        List<Paper> papers = paperRepo.findByWorkIdAndExamNumber(workId, student.getExamNumber());
                        if (!papers.isEmpty()) {
                            for (Paper paper : papers) {
                                Long count = markTaskRepo.countByPaperId(paper.getId());
                                if (count == 0) {
                                    paperRepo.delete(paper);
                                }
                            }
                        }
                    } else {
                        student.setName(dto.getName());
                        student.setAreaCode(dto.getAreaCode());
                        student.setAreaName(dto.getAreaName());
                        student.setExamRoom(dto.getExamRoom());
                        student.setSourceName(dto.getSourceName());
                    }
                    studentRepo.save(student);
                }
                return null;
            }
        });
        return excelErrors;
    }

    /**
     * 清空考生数据 有上传数据无法清空
     *
     * @param workId
     */
    @Transactional
    public void clearStudents(Long workId) {
        Long isExist = paperRepo.countByWorkId(workId);
        if (isExist > 0) {
            throw new RuntimeException("有考生已上传，无法清空");
        }
        studentRepo.deleteByWorkId(workId);
        examQuestionRepo.deleteByWorkId(workId);
    }

    @Transactional
    public Paper savePaper(Student student, Subject subject, boolean isManual) {
        ExamQuestion examQuestion = examQuestionRepo.findByWorkIdAndSubjectAndAreaCode(student.getWorkId(), subject, student.getAreaCode());
        Paper paper = paperRepo.findByWorkIdAndSubjectAndExamNumber(student.getWorkId(), subject, student.getExamNumber());
        if (paper == null) {
            paper = new Paper(student.getWorkId(), null, subject, examQuestion, student, isManual);
        }
        paper.setUploadedOn(new Date());
        paper.setManual(isManual);
        paperRepo.save(paper);
        synchronized (this) {
            String uploadStatus = student.getUploadStatus();
            String replace = subject.toString() + ":1";
            String regx = subject.toString() + ":[0-1]";
            String newStatus = uploadStatus.replaceFirst(regx, replace);
            student.setUploadStatus(newStatus);
            studentRepo.save(student);
        }
        return paper;
    }
}
