package cn.com.qmth.stmms.ms.collect.api;

import cn.com.qmth.stmms.ms.accesscontrol.config.LoginConfig;
import cn.com.qmth.stmms.ms.admin.service.DataUploadService;
import cn.com.qmth.stmms.ms.collect.dto.CollectStuDTO;
import cn.com.qmth.stmms.ms.collect.dto.CollectSubjectDTO;
import cn.com.qmth.stmms.ms.collect.dto.LoginDTO;
import cn.com.qmth.stmms.ms.commons.config.ImageCompressionConfig;
import cn.com.qmth.stmms.ms.commons.config.SystemConfig;
import cn.com.qmth.stmms.ms.commons.utils.image.ImageCompression;
import cn.com.qmth.stmms.ms.core.domain.MarkSubject;
import cn.com.qmth.stmms.ms.core.domain.Paper;
import cn.com.qmth.stmms.ms.core.domain.Student;
import cn.com.qmth.stmms.ms.core.domain.Work;
import cn.com.qmth.stmms.ms.core.repository.MarkSubjectRepo;
import cn.com.qmth.stmms.ms.core.repository.PaperRepo;
import cn.com.qmth.stmms.ms.core.repository.StudentRepo;
import cn.com.qmth.stmms.ms.core.repository.WorkRepo;
import cn.com.qmth.stmms.ms.core.vo.Subject;
import net.sf.json.JSONObject;

import org.apache.commons.codec.digest.DigestUtils;
import org.assertj.core.util.Strings;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.FileCopyUtils;
import org.springframework.web.bind.annotation.*;

import javax.imageio.ImageIO;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import java.awt.image.BufferedImage;
import java.io.*;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Optional;
import java.util.function.Consumer;

/**
 * Created by dizhi on 2016/12/24.
 */
@RestController
@RequestMapping("api")
public class CollectApi {

    @Autowired
    private LoginConfig loginConfig;

    @Autowired
    private WorkRepo workRepo;

    @Autowired
    private StudentRepo studentRepo;

    @Autowired
    private MarkSubjectRepo markSubjectRepo;

    @Autowired
    private DataUploadService dataUploadService;

    @Autowired
    private SystemConfig systemConfig;

    @Autowired
    private ImageCompressionConfig compressionConfig;
    
    @Autowired
    private PaperRepo paperRepo;

    @RequestMapping("user/login")
    public LoginDTO login(@RequestParam String loginname,@RequestParam String password){
        LoginDTO loginDTO = null;
        if(loginname.equals(loginConfig.getLoginName()) && password.equals(loginConfig.getPassword())){
            Work activeWork = workRepo.findByActiveTrue();
            loginDTO = new LoginDTO();
            loginDTO.setExamId(activeWork.getId());
            loginDTO.setExamName(activeWork.getName());
            List<CollectSubjectDTO> collectSubjectDTOs = new ArrayList<>();
            activeWork.getSubjects().forEach(s -> {
                int id = s.getSubject().ordinal()+1;
                CollectSubjectDTO collectSubjectDTO = new CollectSubjectDTO(id,s.getName());
                if(!Strings.isNullOrEmpty(s.getCollectConfig())){
                    JSONObject json = JSONObject.fromObject(s.getCollectConfig());
                    collectSubjectDTO.setCollectConfig(json);
                }

                collectSubjectDTOs.add(collectSubjectDTO);
            });
            loginDTO.setSubjects(collectSubjectDTOs);
        }
        return loginDTO;
    }

    @RequestMapping(value = "exam/students",method = RequestMethod.GET)
    public List<CollectStuDTO> getStudents(){
        List<CollectStuDTO> list = new ArrayList<>();
        Work activeWork = workRepo.findByActiveTrue();
        Consumer<Student> consumer = (s) -> {
            CollectStuDTO collectStuDTO = new CollectStuDTO();
            collectStuDTO.setExamId(activeWork.getId());
            collectStuDTO.setExamNumber(s.getExamNumber());
            collectStuDTO.setName(s.getName());
            collectStuDTO.setSiteCode(s.getAreaName());
            collectStuDTO.setRoomCode(s.getExamNumber());
            list.add(collectStuDTO);
        };
        studentRepo.findByWorkId(activeWork.getId()).forEach(consumer);
        return list;
    }

    @RequestMapping(value = "upload/student/{subjectId}", method = RequestMethod.POST)
    public List<CollectStuDTO> saveStudent(HttpServletRequest request, @PathVariable Integer subjectId,
                                 @RequestBody CollectStuDTO[] uploadStudentArray) {
        Subject subject = Subject.values()[subjectId-1];
        Work activeWork = workRepo.findByActiveTrue();
        List<CollectStuDTO> list = new ArrayList<>();
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH-mm-ss");
        for(CollectStuDTO dto : uploadStudentArray){
            Student student = studentRepo.findByWorkIdAndExamNumber(activeWork.getId(),dto.getExamNumber());
            dataUploadService.savePaper(student,subject,dto.isManual());
            dto.setUploadTime(sdf.format(new Date()));
            list.add(dto);
        }
        return list;
    }
    /**
     * 上传裁切原图并压缩
     * @param workId
     * @param subjectId
     * @param fileName
     * @param request
     * @param response
     * @throws IOException
     */
    @RequestMapping(value = "file/ms-slice/{workId}/{subjectId}/{fileName}", method = { RequestMethod.PUT, RequestMethod.POST })
    public void upload(@PathVariable Long workId,@PathVariable Integer subjectId,
    					@PathVariable String fileName,
                       HttpServletRequest request,HttpServletResponse response) throws IOException{
    	String md5 = request.getHeader("md5");
    	InputStream inputStream = request.getInputStream();
        String sliceMD5 = DigestUtils.md5Hex(inputStream);
        if(!md5.equalsIgnoreCase(sliceMD5)){
        	throw new RuntimeException("图片md5值不一致");
        }
        Student student = studentRepo.findByWorkIdAndExamNumber(workId,fileName);
        Subject subject = Subject.values()[subjectId-1];
        Paper paper = paperRepo.findByWorkIdAndSubjectAndExamNumber(student.getWorkId(), subject,
                student.getExamNumber());
        	//保存遮盖图
        	String savePath = systemConfig.getImageDir() + File.separator + workId + File.separator + subject
        			+ File.separator + student.getAreaCode();
        	File outFile = saveImage(student, inputStream, savePath);
        	// 生成缩略图
        	String thumbDir = systemConfig.getThumbDir() + File.separator + workId + File.separator + subject
        			+ File.separator + student.getAreaCode();
        	File thumb = new File(thumbDir);
        	if (!thumb.exists()) {
        		thumb.mkdirs();
        	}
        	BufferedImage bufferedImage = ImageCompression.compress(outFile, compressionConfig);
        	String thumbFileName = thumbDir + File.separator + student.getExamNumber() + ".jpg";
        	ImageIO.write(bufferedImage, "jpg", new File(thumbFileName));

        paper.setSliceMD5(sliceMD5);
    	paperRepo.save(paper);

//        ImageCompression.imageThumbnail(outFile,new File(thumbFileName),compressionConfig);


    }

	private File saveImage(Student student, InputStream inputStream,
			String savePath) throws IOException, FileNotFoundException {
		File out = new File(savePath);
		if (!out.exists()) {
			out.mkdirs();
		}
		File outFile = new File(savePath + File.separator + student.getExamNumber() + ".jpg");
		FileCopyUtils.copy(inputStream, new FileOutputStream(outFile));
		return outFile;
	}
	
    /**
     * 上传裁切原图并压缩
     * @param workId
     * @param subjectId
     * @param fileName
     * @param request
     * @param response
     * @throws IOException
     */
    @RequestMapping(value = "file/ms-sheet/{workId}/{subjectId}/{fileName}", method = { RequestMethod.PUT, RequestMethod.POST })
    public void uploadsheet(@PathVariable Long workId,@PathVariable Integer subjectId,
    					@PathVariable String fileName,
                       HttpServletRequest request,HttpServletResponse response) throws IOException{
        Student student = studentRepo.findByWorkIdAndExamNumber(workId,fileName);
        Subject subject = Subject.values()[subjectId-1];
        Paper paper = paperRepo.findByWorkIdAndSubjectAndExamNumber(student.getWorkId(), subject,
                student.getExamNumber());
        InputStream inputStream = request.getInputStream();
    	String md5 = request.getHeader("md5");
        String sheetMD5 = DigestUtils.md5Hex(inputStream);
        if(!md5.equalsIgnoreCase(sheetMD5)){
        	throw new RuntimeException("图片md5值不一致");
        }
        //保存原图图
        String savePath = systemConfig.getSheetDir() + File.separator + workId + File.separator + subject
        			+ File.separator + student.getAreaCode();
        saveImage(student, inputStream, savePath);
        paper.setSheetMD5(sheetMD5);
    	paperRepo.save(paper);
    }
    
    @RequestMapping(value = "subject/collect-config", method = RequestMethod.POST)
    public boolean updateConfig(@RequestBody  CollectSubjectDTO collectSubjectDTO){
        Subject subject = Subject.values()[collectSubjectDTO.getSubjectId()-1];
        Work activeWork = workRepo.findByActiveTrue();
        MarkSubject markSubject = markSubjectRepo.findOne(activeWork.getId() + "-" + subject.toString());
        String config = Optional.ofNullable(collectSubjectDTO.getCollectConfig())
                .map(Object::toString)
                .map(s -> s.substring(1,s.length()-1))
                .map(s -> s.replaceAll("\\\\","")).get();
        markSubject.setCollectConfig(config);
        markSubjectRepo.save(markSubject);
        return true;
    }
}
